
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "aes.h"
#include "rota.h"
#include "enc.h"

#define BUFSIZE 256
#define BLOCKSIZE 16


// ------------------- enc P0 Dummy encryption -------------

static void *dummyEnc_create(char *key) {	return malloc(sizeof(int)); } 
static void dummyEnc_destroy(void *encObj) {	free(encObj); }
static void dummyEnc_dec(void *encObj, unsigned char *buf) {}
static void dummyEnc_enc(void *encObj, unsigned char *buf) {}
enc_t enc_dummy_details(void)
{	enc_t enc;
	enc.create = dummyEnc_create;
	enc.destroy = dummyEnc_destroy;
	enc.decBuf = dummyEnc_dec;
	enc.encBuf = dummyEnc_enc;
	enc.encNum = 0;
	enc.bufSize = BUFSIZE;
	return enc;
}


// ------------------- enc P1 and enc P2  AES128 and AES256 -------------

typedef struct
{	aes_context encObj;
	aes_context decObj;
	rota_t *rota;
} encAes_t;


static encAes_t *encAes_create(char *key, int nBitsKey)
{	encAes_t *enc;
	int retVal;
 
// Allocate the structure.
	enc = malloc(sizeof(encAes_t));
	if (enc == NULL)
		return NULL;
 
// Create rota.
	enc->rota = rota_create(key);
	if (enc-> rota == NULL)
	{	free(enc);
		return NULL;
	}
 
// Create the context for encryption.
	if (aes_setkey_enc(&enc->encObj, rota_digest(enc->rota), nBitsKey) != 0)
	{	rota_destroy(enc->rota);
		free(enc);
		return NULL;
	}
 
// Create the context for decryption.
	if (aes_setkey_dec(&enc->decObj, rota_digest(enc->rota), nBitsKey) != 0)
	{	rota_destroy(enc->rota);
		free(enc);
		return NULL;
	}
 
	return enc;
}


static void encAes_destroy(encAes_t *enc)
{	rota_destroy(enc->rota);
	free(enc);
}


static void encAes_encrypt(encAes_t *enc, unsigned char *buffer)
{	int i;
	unsigned char *block, *end;
	unsigned char tempBlock[BLOCKSIZE];
 
// Mask, so encrypted blocks are not identical even if the plaintext blocks are identical.
	rota_reset(enc->rota);
	for (i=0; i<BUFSIZE; i++)
		buffer[i] ^= rota_mask(enc->rota, buffer[i]); 
 
// Encrypt each block within the buffer.
	end = buffer + BUFSIZE;
	for (block=buffer; block<end; block+=BLOCKSIZE)
	{	aes_crypt_ecb(&enc->encObj, AES_ENCRYPT, block, tempBlock);
		memcpy(block, tempBlock, BLOCKSIZE);
	}
}


static void encAes_decrypt(encAes_t *enc, unsigned char *buffer)
{	int i;
	unsigned char *block, *end;
	unsigned char tempBlock[BLOCKSIZE];
 
// Encrypt each block within the buffer.
	end = buffer + BUFSIZE;
	for (block=buffer; block<end; block+=BLOCKSIZE)
	{	aes_crypt_ecb(&enc->decObj, AES_DECRYPT, block, tempBlock);
		memcpy(block, tempBlock, BLOCKSIZE);
	}
 
// Unmask.
	rota_reset(enc->rota);
	for (i=0; i<BUFSIZE; i++)
		buffer[i] = rota_unmask(enc->rota, buffer[i]); 
}


static void *aes128_create(char *key) { return (void*)encAes_create(key, 128); }
static void *aes256_create(char *key) { return (void*)encAes_create(key, 256); }
static void aesEnc_destroy(void *enc) { encAes_destroy((encAes_t*)enc); }
static void aesEnc_enc(void *enc, unsigned char *buf) { encAes_encrypt((encAes_t*)enc, buf); }
static void aesEnc_dec(void *enc, unsigned char *buf) { encAes_decrypt((encAes_t*)enc, buf); }

enc_t enc_aes128_details(void)
{	enc_t enc;
	enc.create = aes128_create;
	enc.destroy = aesEnc_destroy;
	enc.encBuf = aesEnc_enc;
	enc.decBuf = aesEnc_dec;
	enc.encNum = 1;
	enc.bufSize = BUFSIZE;
	return enc;
}

enc_t enc_aes256_details(void)
{	enc_t enc;
	enc.create = aes256_create;
	enc.destroy = aesEnc_destroy;
	enc.encBuf = aesEnc_enc;
	enc.decBuf = aesEnc_dec;
	enc.encNum = 2;
	enc.bufSize = BUFSIZE;
	return enc;
}


// ------------------- enc P3 triple AES -------------

typedef struct
{	aes_context encObj1, encObj2, encObj3;
	aes_context decObj1, decObj2, decObj3;
	rota_t *rota;
} encAes3_t;


static encAes3_t *encAes3_create(char *key)
{	unsigned char *digest;
	encAes3_t *enc;
	int retVal;
 
// Allocate the structure.
	enc = malloc(sizeof(encAes3_t));
	if (enc == NULL)
		return NULL;
 
// Create rota.
	enc->rota = rota_create(key);
	if (enc-> rota == NULL)
	{	free(enc);
		return NULL;
	}
	digest = rota_digest(enc->rota);
 
// Create the context for encryption.
	if (aes_setkey_enc(&enc->encObj1, digest, 128) != 0)
	{	rota_destroy(enc->rota);
		free(enc);
		return NULL;
	}
	if (aes_setkey_enc(&enc->encObj2, digest+BLOCKSIZE, 128) != 0)
	{	rota_destroy(enc->rota);
		free(enc);
		return NULL;
	}
	if (aes_setkey_enc(&enc->encObj3, digest+2*BLOCKSIZE, 128) != 0)
	{	rota_destroy(enc->rota);
		free(enc);
		return NULL;
	}
 
// Create the context for decryption.
	if (aes_setkey_dec(&enc->decObj1, digest, 128) != 0)
	{	rota_destroy(enc->rota);
		free(enc);
		return NULL;
	}
	if (aes_setkey_dec(&enc->decObj2, digest+BLOCKSIZE, 128) != 0)
	{	rota_destroy(enc->rota);
		free(enc);
		return NULL;
	}
	if (aes_setkey_dec(&enc->decObj3, digest+2*BLOCKSIZE, 128) != 0)
	{	rota_destroy(enc->rota);
		free(enc);
		return NULL;
	}
 
	return enc;
}


static void encAes3_destroy(encAes3_t *enc)
{	rota_destroy(enc->rota);
	free(enc);
}


static void encAes3_encrypt(encAes3_t *enc, unsigned char *buffer)
{	typedef unsigned char block_t[BLOCKSIZE];
	int i, j;
	unsigned char *tempBlock, *block, *end;
	block_t *buf = (block_t*) buffer;
	block_t tempBuf[BLOCKSIZE];
 
// Mask, so encrypted blocks are not identical even if the plaintext blocks are identical.
	rota_reset(enc->rota);
	tempBlock = (unsigned char*)tempBuf;
	for (i=0; i<BUFSIZE; i++)
		tempBlock[i] = rota_mask(enc->rota, buffer[i]); 
 
// Encrypt each block to the buffer.
	tempBlock = (unsigned char*)tempBuf;
	end = buffer + BUFSIZE;
	for (block=buffer; block<end; block+=BLOCKSIZE, tempBlock+=BLOCKSIZE)
		aes_crypt_ecb(&enc->encObj1, AES_ENCRYPT, tempBlock, block);
 
// Scatter the encrypted bytes to the temporary buffer.
	for (i=0; i<BLOCKSIZE; i++)
		for (j=0; j<BLOCKSIZE; j++)
			tempBuf[i][j] = buf[j][i];
	
// Encrypt each block to the buffer.
	tempBlock = (unsigned char*) tempBuf;
	end = buffer + BUFSIZE;
	for (block=buffer; block<end; block+=BLOCKSIZE, tempBlock+=BLOCKSIZE)
		aes_crypt_ecb(&enc->encObj2, AES_ENCRYPT, tempBlock, block);
 
// Scatter the encrypted bytes to the temporary buffer.
	for (i=0; i<BLOCKSIZE; i++)
		for (j=0; j<BLOCKSIZE; j++)
			tempBuf[i][j] = buf[j][i];
	
// Encrypt each block to the buffer.
	tempBlock = (unsigned char*) tempBuf;
	end = buffer + BUFSIZE;
	for (block=buffer; block<end; block+=BLOCKSIZE, tempBlock+=BLOCKSIZE)
		aes_crypt_ecb(&enc->encObj3, AES_ENCRYPT, tempBlock, block);
}


static void encAes3_decrypt(encAes3_t *enc, unsigned char *buffer)
{	typedef unsigned char block_t[BLOCKSIZE];
	int i, j;
	unsigned char *tempBlock, *block, *end;
	block_t *buf = (block_t*) buffer;
	block_t tempBuf[BLOCKSIZE];
 
// Encrypt each block from the buffer.
	tempBlock = (unsigned char*) tempBuf;
	end = buffer + BUFSIZE;
	for (block=buffer; block<end; block+=BLOCKSIZE, tempBlock+=BLOCKSIZE)
		aes_crypt_ecb(&enc->decObj3, AES_DECRYPT, block, tempBlock);
 
// Gather the decrypted bytes from the temporary buffer.
	for (i=0; i<BLOCKSIZE; i++)
		for (j=0; j<BLOCKSIZE; j++)
			buf[i][j] = tempBuf[j][i];
	
// Encrypt each block from the buffer.
	tempBlock = (unsigned char*) tempBuf;
	end = buffer + BUFSIZE;
	for (block=buffer; block<end; block+=BLOCKSIZE, tempBlock+=BLOCKSIZE)
		aes_crypt_ecb(&enc->decObj2, AES_DECRYPT, block, tempBlock);
 
// Gather the decrypted bytes from the temporary buffer.
	for (i=0; i<BLOCKSIZE; i++)
		for (j=0; j<BLOCKSIZE; j++)
			buf[i][j] = tempBuf[j][i];
	
// Encrypt each block from the buffer.
	tempBlock = (unsigned char*) tempBuf;
	end = buffer + BUFSIZE;
	for (block=buffer; block<end; block+=BLOCKSIZE, tempBlock+=BLOCKSIZE)
		aes_crypt_ecb(&enc->decObj1, AES_DECRYPT, block, tempBlock);
 
// Unmask, so encrypted blocks are not identical even if the plaintext blocks are identical.
	rota_reset(enc->rota);
	tempBlock = (unsigned char*)tempBuf;
	for (i=0; i<BUFSIZE; i++)
		buffer[i] = rota_unmask(enc->rota, tempBlock[i]); 
}


static void *aes3_create(char *key) { return (void*)encAes3_create(key); }
static void aes3_destroy(void *enc) { encAes3_destroy((encAes3_t*)enc); }
static void aes3_enc(void *enc, unsigned char *buf) { encAes3_encrypt((encAes3_t*)enc, buf); }
static void aes3_dec(void *enc, unsigned char *buf) { encAes3_decrypt((encAes3_t*)enc, buf); }

enc_t enc_aes3_details(void)
{	enc_t enc;
	enc.create = aes3_create;
	enc.destroy = aes3_destroy;
	enc.encBuf = aes3_enc;
	enc.decBuf = aes3_dec;
	enc.encNum = 3;
	enc.bufSize = BUFSIZE;
	return enc;
}



